ALLOWED_ATTRIBUTES = {
    '*': ['class', 'style'],
    'a': ['href', 'title']
}

ALLOWED_CSS_PROPERTIES = [
    'color'
]

ALLOWED_PROTOCOLS = [
    'https',
    'data',
]

ALLOWED_STYLES = ALLOWED_CSS_PROPERTIES

ALLOWED_TAGS = [
    'a',
    'li',
    'ul'
]
