# Generated by Django 3.2.4 on 2021-06-15 12:39

from django.db import migrations, models
import django_bleach.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20)),
                ('biography', django_bleach.models.BleachField(max_length=100, verbose_name='Person biography')),
            ],
        ),
    ]
